import pygame
from pygame.locals import *

import menu
import state
import resources
import random
import text
import tilelayer
import pickle
import factory

class Editor(state.State):
    """Actual game state."""
    def __init__(self, engine):
        super(Editor, self).__init__(engine)
        self.tileset = resources.tilesets["foreground"]
        self.back_tileset = resources.tilesets["background"]

        self.background = tilelayer.Tilelayer(128, 31, self.back_tileset, "back", 0.8)
        self.tiles = tilelayer.Tilelayer(128, 31, self.tileset, "front")

        factory.load_factory()
        
        self.entities = []
        self.entity_selection = None

        self.camera_pos = Rect(0, 0, self.engine.width, self.engine.height)
        self.tick = 0

        self.selection = 0
        self.rightside_y = 0
        self.text = [text.Text(480 + 10 + self.tileset.get_size()[0] + 5,
                               i * self.tileset.get_size()[1] + 5,
                               font=engine.s_font, text=name)
                     for i, name in enumerate(self.tileset.tile_names + self.back_tileset.tile_names)]
        self.foreground_tile_count = len(self.tileset.tile_names)
        self.text[self.selection].set_color((255, 0, 0))
        
    def display(self, screen):
        clip = screen.get_clip()
        screen.set_clip((0, 0, 480, 480))

        self.background.display(screen, self.camera_pos)
        self.tiles.display(screen, self.camera_pos)

        for ent, _, _ in self.entities:
            ent.display(screen, self.camera_pos, 0)

        if self.entity_selection is not None:
            factory.factory_list[self.entity_selection][1].display(screen, pygame.mouse.get_pos(), 0)

        screen.set_clip((480, 0, 160, 480))
        for i, text in enumerate(self.text):
            if i < self.foreground_tile_count:
                self.tileset.display(screen, i,
                                     (480 + 10, i * self.tileset.get_size()[1] + self.rightside_y))
            else:
                i_adj = i - self.foreground_tile_count
                self.back_tileset.display(screen, i_adj,
                                          (480 + 10, i * self.tileset.get_size()[1] + self.rightside_y))
            text.move(text.pos.x, i * self.tileset.get_size()[1] + self.rightside_y + 5)
            text.display(screen)
        screen.set_clip(clip)
        
    def update(self):
        self.tick += 1
        keys = pygame.key.get_pressed()
        if keys[K_LEFT] or keys[ord('a')]:
            self.camera_pos.x -= 12
        if keys[K_RIGHT] or keys[ord('d')]:
            self.camera_pos.x += 12
        if keys[K_UP] or keys[ord('w')]:
            self.camera_pos.y -= 12
        if keys[K_DOWN] or keys[ord('s')]:
            self.camera_pos.y += 12
        if pygame.mouse.get_pressed()[0] and self.entity_selection is None:
            mx, my = pygame.mouse.get_pos()
            if mx < 480:
                x = (mx + self.camera_pos.x) / self.tileset.get_size()[0]
                y = (my + self.camera_pos.y) / self.tileset.get_size()[1]
                if x >= 0 and y >= 0 and x < self.tiles.width and y < self.tiles.height:
                    if self.selection < self.foreground_tile_count:
                        self.tiles.set_tile(x, y, self.selection)
                    else:
                        x = (int(mx + self.camera_pos.x * self.background.camera_mult)
                             / self.back_tileset.get_size()[0])
                        y = (int(my + self.camera_pos.y * self.background.camera_mult)
                             / self.back_tileset.get_size()[1])
                        self.background.set_tile(x, y, self.selection - self.foreground_tile_count)
            else:
                self.select((my - self.rightside_y) / self.tileset.get_size()[1])
    
    def handle_event(self, event):
        if event.type == KEYDOWN:
            if event.key == K_ESCAPE:
                self.engine.set_state(menu.Menu(self.engine))
            elif event.key == ord('o'):
                with open("level.dump", "w") as f:
                    pickle.dump(self.tiles.tiles, f)
                    pickle.dump(self.background.tiles, f)
                    pickle.dump([(name, pos) for _, name, pos in self.entities], f)
            elif event.key == ord('l'):
                with open("level.dump") as f:
                    self.tiles.tiles = pickle.load(f)
                    self.background.tiles = pickle.load(f)
                    self.entities = [(factory.factory(name, pos, self.tiles), name, pos)
                                     for name, pos in pickle.load(f)]
            elif event.key >= ord('1') and event.key <= ord('9'):
                self.select(int(chr(event.key)) - 1)
            elif event.key == ord('e'):
                if self.entity_selection is None:
                    self.entity_selection = 0
                else:
                    self.entity_selection = None
            elif event.key == K_BACKSPACE:
                mx, my = pygame.mouse.get_pos()
                mx += self.camera_pos.x
                my += self.camera_pos.y
                for t in self.entities:
                    ent, _, _ = t
                    if ent.pos.collidepoint(mx, my):
                        self.entities.remove(t)
                        break
        elif event.type == MOUSEBUTTONDOWN:
            if event.button == 4: # Scroll up
                if self.entity_selection is not None:
                    self.entity_selection += 1
                    self.entity_selection %= len(factory.factory_list)
                else:
                    self.rightside_y -= 8
            elif event.button == 5: # Scroll down
                if self.entity_selection is not None:
                    self.entity_selection -= 1
                    self.entity_selection %= len(factory.factory_list)
                else:
                    self.rightside_y += 8
            elif self.entity_selection is not None:
                new_ent = factory.factory_list[self.entity_selection]
                mx, my = pygame.mouse.get_pos()
                mx += self.camera_pos.x
                my += self.camera_pos.y
                self.entities.append((factory.factory(new_ent[0], (mx, my), self.tiles),
                                      new_ent[0],
                                      (mx, my)))
                
    def select(self, new_selection):
        self.text[self.selection].set_color((255, 255, 255))
        if new_selection >= 0 and new_selection < len(self.text):
            self.selection = new_selection
        self.text[self.selection].set_color((255, 0, 0))
