import pygame
from pygame.locals import *

import win
import died
import menu
import state
import resources
import random
import bird
import tilelayer
import text
import pickle
import item
import factory

class Game(state.State):
    """Actual game state."""
    def __init__(self, engine):
        super(Game, self).__init__(engine)
        self.tileset = resources.tilesets["foreground"]
        self.back_tileset = resources.tilesets["background"]

        self.background = tilelayer.Tilelayer(128, 31, self.back_tileset, "back", 0.8)
        self.tiles = tilelayer.Tilelayer(128, 31, self.tileset, "front")
        with open("level.dump") as f:
            self.tiles.tiles = pickle.load(f)
            self.background.tiles = pickle.load(f)
            self.level_entity_list = pickle.load(f)

        self.camera_pos = Rect(0, 0, self.engine.width, self.engine.height)
        self.tick = 0

        self.player = bird.Butterfly((128, 256), self.tiles)
        self.entities = set([self.player])
        self.background_entities = set()
        self.ai_entities = set()
        self.killer_entities = set()
        self.goal_entities = set()
        self.entities_to_remove = set()
        self.liftable_entities = set()

        for name, pos in self.level_entity_list:
            new_ent = factory.factory(name, pos, self.tiles)

            if name in factory.background_entity:
                self.background_entities.add(new_ent)
            else:
                self.entities.add(new_ent)
            if name in factory.ai_entity:
                self.ai_entities.add(new_ent)
            if name in factory.kills_player_entity:
                self.killer_entities.add(new_ent)
            if name in factory.goal_entity:
                self.goal_entities.add(new_ent)
            if name in factory.liftable_entity:
                self.liftable_entities.add(new_ent)

        self.hud = text.Text(0, 440 + 5, font=engine.m_font, text="Arrows/WASD = move   Space = grab/drop")

    def remove_entity(self, entity):
        self.entities_to_remove.add(entity)
        
    def display(self, screen):
        clip = screen.get_clip()
        screen.set_clip((0, 0, 640, 440))

        self.background.display(screen, self.camera_pos)
        for entity in self.background_entities:
            entity.display(screen, self.camera_pos, self.tick)

        self.tiles.display(screen, self.camera_pos)

        for entity in self.entities:
            entity.display(screen, self.camera_pos, self.tick)

        screen.set_clip((0, 440, 640, 80))
        self.hud.display(screen)
        screen.set_clip(clip)
        
    def update(self):
        if self.engine.play_music and not pygame.mixer.music.get_busy():
            pygame.mixer.music.load(resources.music["loop1"])
            pygame.mixer.music.play(random.randint(0,3))
            pygame.mixer.music.queue(random.choice([resources.music["loop2"], resources.music["loop3"]]))
            
        self.tick += 1
        keys = pygame.key.get_pressed()

        if keys[K_LEFT] or keys[ord('a')]:
            self.player.move(-5, 0)
        if keys[K_RIGHT] or keys[ord('d')]:
            self.player.move(5, 0)
        if keys[K_UP] or keys[ord('w')]:
            self.player.move(0, -10)
            
        for entity in self.ai_entities:
            entity.ai()

        for entity in self.entities:
            entity.check_collision(self.tiles)
            entity.update()

        for entity in self.background_entities:
            entity.update()

        won = True
        for entity in self.goal_entities:
            if entity.pos.x > 300:
                won = False
                break
        if won:
            self.engine.set_state(win.Win(self.engine, self))
            
        for entity in self.killer_entities:
            if self.player.pos.colliderect(entity.pos):
                self.engine.set_state(died.Died(self.engine, self))
            
        for entity in self.entities_to_remove:
            self.ai_entities.discard(entity)
            self.entities.discard(entity)
        self.entities_to_remove.clear()

        self.maintain_camera()

    def maintain_camera(self):
        """Maintain the camera position.

        Should have the player in the centre, and not go beyond the bounds of the level.
        """

        # Keep camera on the player
        if self.player.pos.x - self.camera_pos.x < self.engine.width / 3:
            self.camera_pos.x = self.player.pos.x - self.engine.width / 3
        if self.player.pos.x - self.camera_pos.x > self.engine.width * 2 / 3:
            self.camera_pos.x = self.player.pos.x - self.engine.width * 2 / 3
        if self.player.pos.y - self.camera_pos.y < self.engine.height / 3:
            self.camera_pos.y = self.player.pos.y - self.engine.height / 3
        if self.player.pos.y - self.camera_pos.y > self.engine.height * 2 / 3:
            self.camera_pos.y = self.player.pos.y - self.engine.height * 2 / 3 

        # Don't let camera go beyond the level
        # (Since this happens after the above, it overrides it)
        w, h = self.tiles.get_total_size()
        if self.camera_pos.x + self.engine.width > w:
            self.camera_pos.x = w - self.engine.width

        if self.camera_pos.y + self.engine.height > h:
            self.camera_pos.y = h - self.engine.height

        if self.camera_pos.x < 0:
            self.camera_pos.x = 0

        if self.camera_pos.y < 0:
            self.camera_pos.y = 0

    def handle_event(self, event):
        if event.type == KEYDOWN:
            if event.key == K_ESCAPE:
                pygame.mixer.music.stop()
                self.engine.set_state(menu.Menu(self.engine))

            # TODO: eventually remove this
            if event.key == ord('t'):
                if self.player.__class__ == bird.Butterfly:
                    self.remove_entity(self.player)
                    self.player = bird.Bird((self.player.pos.x, self.player.pos.y), self.tiles)
                    self.entities.add(self.player)
                elif self.player.__class__ == bird.Bird:
                    self.remove_entity(self.player)
                    self.player = bird.Butterfly((self.player.pos.x, self.player.pos.y), self.tiles)
                    self.entities.add(self.player)
                    
            if event.key == K_SPACE:
                picked_up = False
                for item in self.liftable_entities:
                    if self.player.is_touching(item) and not picked_up:
                        item.holder = self.player
                        picked_up = True
                    elif item.holder is not None:
                        item.holder = None
